# import sys
# sys.path.append("../")
import audiostack
import os

audiostack.api_base = "https://staging-v2.api.audio"

audiostack.api_key = os.environ["AUDIO_STACK_DEV_KEY"]

scriptText = """
<as:section name="intro" soundsegment="intro"> 
    hello enjoy a coffee with <as:placeholder id="name">name</as:placeholder>
</as:section>
"""

script = audiostack.Content.Script.create(scriptText=scriptText)
print("response from creating script", script.response)
scriptId = script.scriptId

# create one tts resource
tts = audiostack.Speech.TTS.create(scriptItem=script, voice="sara", audience={"name" : "Sam"})
print(tts.speechId)

tts.download(fileName="name")


from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes
from audiostack.helpers.api_item import APIResponseItem
from audiostack.helpers.api_list import APIResponseList

from typing import Union


class Sound:
    interface = RequestInterface(family="production/sound")

    # ----------------------------------------- TEMPLATE -----------------------------------------
    class Template:
        class Item(APIResponseItem):
            def __init__(self, response) -> None:
                super().__init__(response)

                if "template" in self.data:  #
                    self.data = self.data["template"]

                self.templateName = self.data["templateName"]
                self.collections = self.data["collections"]
                self.genre = self.data["genre"]
                self.description = self.data["description"]
                self.tempo = self.data["tempo"]
                self.tags = self.data["tags"]

        class List(APIResponseList):
            def __init__(self, response, list_type) -> None:
                super().__init__(response, list_type)

            def resolve_item(self, list_type, item):
                if list_type == "templates":
                    return Sound.Template.Item({"data": item})
                else:
                    raise Exception()

        @staticmethod
        def list(
            tags: Union[str, list] = "",
            contents: Union[str, list] = "",
            collections: Union[str, list] = "",
            genre: str = "",
            tempo: str = "",
            type: str = "all",
        ) -> list:
            if type not in ["all", "custom", "standard"]:
                raise Exception(
                    "Invalid type supplied, should be 'all', 'custom', 'standard'"
                )

            query_params = {
                "tags": tags,
                "contents": contents,
                "collections": collections,
                "genre": genre,
                "tempo": tempo,
                "type": type,
            }
            r = Sound.interface.send_request(
                rtype=RequestTypes.GET, route="template", query_parameters=query_params
            )
            return Sound.Template.List(r, list_type="templates")

        def create(templateName: str, description: str = ""):
            body = {"templateName": templateName, "description": description}
            r = Sound.interface.send_request(
                rtype=RequestTypes.POST, route="template", json=body
            )
            return Sound.Template.Item(r)

        def delete(templateName: str):
            r = Sound.interface.send_request(
                rtype=RequestTypes.DELETE,
                route="template",
                path_parameters=templateName,
            )
            return APIResponseItem(r)

        def update(
            templateName: str,
            description: str = "",
            genre: str = "",
            collections: list = None,
            tags: list = None,
        ):
            body = {
                "templateName": templateName,
                "description": description,
                "genre": genre,
                "collections": collections,
                "tags": tags,
            }
            r = Sound.interface.send_request(
                rtype=RequestTypes.PUT, route="template", json=body
            )
            return Sound.Template.Item(r)

    # ----------------------------------------- TEMPLATE SEGMENT -----------------------------------------
    class Segment:
        def create(mediaId: str, templateName: str, soundSegmentName: str):
            segment = {
                "templateName": templateName,
                "segmentName": soundSegmentName,
                "mediaId": mediaId,
            }
            r = Sound.interface.send_request(
                rtype=RequestTypes.POST, route="segment", json=segment
            )
            return Sound.Template.Item(r)

    # ----------------------------------------- TEMPLATE PARAMETERS -----------------------------------------
    class Parameter:
        @staticmethod
        def get() -> dict:
            r = Sound.interface.send_request(rtype=RequestTypes.GET, route="parameter")
            return APIResponseItem(r)



from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes
from audiostack.helpers.api_item import APIResponseItem
from audiostack.helpers.api_list import APIResponseList


class Mix:
    interface = RequestInterface(family="production")

    class Item(APIResponseItem):
        def __init__(self, response) -> None:
            super().__init__(response)
            self.productionId = self.data["productionId"]

        def download(self, fileName="", path="./") -> None:
            sections = self.data["files"]
            for i, s in enumerate(sections):
                format = s["format"]
                if not fileName:
                    full_name = f"default_mix.{format}"
                else:
                    full_name = f"{fileName}.{format}"

                RequestInterface.download_url(
                    s["url"], destination=path, name=full_name
                )

        def delete(self):
            return Mix.delete(self.productionId)

    class List(APIResponseList):
        def __init__(self, response, list_type) -> None:
            super().__init__(response, list_type)

        def resolve_item(self, list_type, item):
            if list_type == "productionIds":
                return Mix.Item({"data": item})
            else:
                raise Exception()

    @staticmethod
    def create(
        speechId="",
        speechItem=None,
        soundTemplate: str = "",
        forceLength: float = 0.0,  # a value of 0 means no force length
        mediaFiles: dict = {},
        sectionProperties: dict = {},
        masteringPreset: str = "",
        acousticSpace: str = "",
        callbackUrl: str = "",
        public: bool = False,
    ) -> Item:
        if speechId and speechItem:
            raise Exception("speechId or scriptItem should be supplied not both")
        if not (speechId or speechItem):
            raise Exception("speechId or scriptItem should be supplied")
        if speechItem:
            speechId = speechItem.speechId

        if not isinstance(soundTemplate, str):
            raise Exception("soundTemplate argument should be a string")
        if not isinstance(forceLength, float):
            raise Exception("forceLength should be a float")
        if not isinstance(masteringPreset, str):
            raise Exception("masteringPreset should be a string")
        if not isinstance(acousticSpace, str):
            raise Exception("acousticSpace should be a string")

        body = {
            "speechId": speechId,
            "soundTemplate": soundTemplate,
            "forceLength": forceLength,
            "mediaFiles": mediaFiles,
            "sectionProperties": sectionProperties,
            "masteringPreset": masteringPreset,
            "acousticSpace": acousticSpace,
            "callbackUrl": callbackUrl,
            "public": public,
        }

        r = Mix.interface.send_request(rtype=RequestTypes.POST, route="mix", json=body)
        if r["statusCode"] == 202:
            print("Response in progress please wait...")
            return Mix.get(Mix.Item(r).productionId)
        
        return Mix.Item(r)

    @staticmethod
    def get(productionId: str) -> Item:
        r = Mix.interface.send_request(
            rtype=RequestTypes.GET, route="mix", path_parameters=productionId
        )
        return Mix.Item(r)

    @staticmethod
    def delete(productionId: str) -> str:
        r = Mix.interface.send_request(
            rtype=RequestTypes.DELETE, route="mix", path_parameters=productionId
        )
        return APIResponseItem(r)

    @staticmethod
    def list(
        projectName="", moduleName: str = "", scriptName: str = "", scriptId: str = ""
    ) -> list:
        query_params = {
            "projectName": projectName,
            "moduleName": moduleName,
            "scriptName": scriptName,
            "scriptId": scriptId,
        }

        r = Mix.interface.send_request(
            rtype=RequestTypes.GET, route="mixes", query_parameters=query_params
        )
        return Mix.List(r, list_type="productionIds")
